"use strict";
/* --------------------------------------------------------------------------------------------
 * Copyright (c) Microsoft Corporation. All rights reserved.
 * Licensed under the MIT License. See License.txt in the project root for license information.
 * ------------------------------------------------------------------------------------------ */
Object.defineProperty(exports, "__esModule", { value: true });
exports.boolean = boolean;
exports.nullOrUndefined = nullOrUndefined;
exports.string = string;
const toString = Object.prototype.toString;
function boolean(value) {
    return value === true || value === false;
}
function nullOrUndefined(value) {
    return value === null || value === undefined;
}
function string(value) {
    return toString.call(value) === '[object String]';
}
//# sourceMappingURL=is.js.map