import Range = require("../classes/range");
import SemVer = require("../classes/semver");
import semver = require("../index");

/**
 * Return true if version is less than all the versions possible in the range.
 */
declare function ltr(
    version: string | SemVer,
    range: string | Range,
    optionsOrLoose?: boolean | semver.RangeOptions,
): boolean;

export = ltr;
