export declare function boolean(value: any): value is boolean;
export declare function nullOrUndefined(value: any): value is null | undefined;
export declare function string(value: any): value is string;
