import semver = require("../index");

/**
 * Returns cleaned (removed leading/trailing whitespace, remove '=v' prefix) and parsed version, or null if version is invalid.
 */
declare function clean(version: string, optionsOrLoose?: boolean | semver.Options): string | null;

export = clean;
